@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2013-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -22,12 +22,11 @@
 
 #include <time.h>
 
-#define SECURE_ENV_PASSPHRASE       "WEECHAT_PASSPHRASE"
-#define SECURE_SALT_DEFAULT         "WeeChat!"
-#define SECURE_DATA_PASSPHRASE_FLAG "__passphrase__"
-#define SECURE_SALT_SIZE            8
-#define SECURE_TOTP_MIN_DIGITS      4
-#define SECURE_TOTP_MAX_DIGITS      10
+#define SECURE_ENV_PASSPHRASE        "WEECHAT_PASSPHRASE"
+#define SECURE_PASSPHRASE_MAX_LENGTH 4096
+#define SECURE_SALT_DEFAULT          "WeeChat!"
+#define SECURE_DATA_PASSPHRASE_FLAG  "__passphrase__"
+#define SECURE_SALT_SIZE             8
 
 enum t_secure_config_hash_algo
 {
@@ -63,10 +62,6 @@ extern int secure_decrypt_data (const char *buffer, int length_buffer,
                                 const char *passphrase, char **decrypted,
                                 int *length_decrypted);
 extern int secure_decrypt_data_not_decrypted (const char *passphrase);
-extern char *secure_totp_generate (const char *secret, time_t totp_time,
-                                   int digits);
-extern int secure_totp_validate (const char *secret, time_t totp_time,
-                                 int window, const char *otp);
 extern int secure_init ();
 extern void secure_end ();
 
