@@ -1,7 +1,7 @@
 /*
  * wee-upgrade.c - save/restore session data of WeeChat core
  *
- * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,7 +32,7 @@
 #include "wee-upgrade.h"
 #include "wee-hook.h"
 #include "wee-infolist.h"
-#include "wee-secure.h"
+#include "wee-secure-buffer.h"
 #include "wee-string.h"
 #include "wee-util.h"
 #include "../gui/gui-buffer.h"
@@ -446,6 +446,12 @@ upgrade_weechat_read_buffer (struct t_infolist *infolist)
     /* full name */
     gui_buffer_build_full_name (ptr_buffer);
 
+    /* old full name */
+    if (ptr_buffer->old_full_name)
+        free (ptr_buffer->old_full_name);
+    str = infolist_string (infolist, "old_full_name");
+    ptr_buffer->old_full_name = (str) ? strdup (str) : NULL;
+
     /* short name */
     if (ptr_buffer->short_name)
         free (ptr_buffer->short_name);
