@@ -17,7 +17,7 @@
  *
  * weechat.c - WeeChat main functions
  *
- * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -32,17 +32,19 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <unistd.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <getopt.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <time.h>
@@ -65,6 +67,7 @@
 #include "wee-network.h"
 #include "wee-proxy.h"
 #include "wee-secure.h"
+#include "wee-secure-config.h"
 #include "wee-string.h"
 #include "wee-upgrade.h"
 #include "wee-utf8.h"
@@ -79,6 +82,10 @@
 #include "../plugins/plugin.h"
 #include "../plugins/plugin-api.h"
 
+/* debug command line options */
+#define OPTION_NO_DLCLOSE 1000
+#define OPTION_NO_GNUTLS  1001
+#define OPTION_NO_GCRYPT  1002
 
 int weechat_headless = 0;              /* 1 if running headless (no GUI)    */
 int weechat_debug_core = 0;            /* debug level for core              */
@@ -93,6 +100,8 @@ volatile sig_atomic_t weechat_quit = 0;   /* = 1 if quit request from user  */
 volatile sig_atomic_t weechat_quit_signal = 0; /* signal received,          */
                                        /* WeeChat must quit                 */
 char *weechat_home = NULL;             /* home dir. (default: ~/.weechat)   */
+int weechat_home_temp = 0;             /* 1 if using a temporary home       */
+int weechat_home_delete_on_exit = 0;   /* 1 if home is deleted on exit      */
 int weechat_locale_ok = 0;             /* is locale OK?                     */
 char *weechat_local_charset = NULL;    /* example: ISO-8859-1, UTF-8        */
 int weechat_server_cmd_line = 0;       /* at least 1 server on cmd line     */
@@ -105,6 +114,7 @@ int weechat_no_gcrypt = 0;             /* remove init/deinit of gcrypt      */
                                        /* (useful with valgrind)            */
 struct t_weelist *weechat_startup_commands = NULL; /* startup commands      */
                                                    /* (option -r)           */
+int weechat_auto_connect = 1;          /* auto-connect to servers           */
 
 
 /*
@@ -151,6 +161,9 @@ weechat_display_usage ()
           "(default: ~/.weechat)\n"
           "                           (environment variable WEECHAT_HOME is "
           "read if this option is not given)\n"
+          "  -t, --temp-dir           create a temporary WeeChat home"
+          "directory and delete it on exit\n"
+          "                           (incompatible with option \"-d\")\n"
           "  -h, --help               display this help\n"
           "  -l, --license            display WeeChat license\n"
           "  -p, --no-plugin          don't load any plugin at startup\n"
@@ -182,6 +195,16 @@ weechat_display_usage ()
               "WeeChat is blocking and does not run in background)\n"));
         string_fprintf (stdout, "\n");
     }
+
+    /* debug options */
+    string_fprintf (
+        stdout,
+        _("Debug options (for tools like valgrind, DO NOT USE IN PRODUCTION):\n"
+          "      --no-dlclose         do not call function dlclose after "
+          "plugins are unloaded\n"
+          "      --no-gnutls          disable init/deinit of gnutls\n"
+          "      --no-gcrypt          disable init/deinit of gcrypt\n"));
+    string_fprintf (stdout, "\n");
 }
 
 /*
@@ -193,137 +216,145 @@ weechat_display_usage ()
 void
 weechat_parse_args (int argc, char *argv[])
 {
-    int i;
+    int opt;
+    struct option long_options[] = {
+        /* standard options */
+        { "no-connect",  no_argument,       NULL, 'a'  },
+        { "colors",      no_argument,       NULL, 'c'  },
+        { "dir",         required_argument, NULL, 'd'  },
+        { "temp-dir",    no_argument,       NULL, 't'  },
+        { "help",        no_argument,       NULL, 'h'  },
+        { "license",     no_argument,       NULL, 'l'  },
+        { "no-plugin",   no_argument,       NULL, 'p'  },
+        { "plugins",     required_argument, NULL, 'P'  },
+        { "run-command", required_argument, NULL, 'r'  },
+        { "no-script",   no_argument,       NULL, 's'  },
+        { "upgrade",     no_argument,       NULL, 'u'  },
+        { "version",     no_argument,       NULL, 'v'  },
+        /* debug options */
+        { "no-dlclose",  no_argument,       NULL, OPTION_NO_DLCLOSE },
+        { "no-gnutls",   no_argument,       NULL, OPTION_NO_GNUTLS  },
+        { "no-gcrypt",   no_argument,       NULL, OPTION_NO_GCRYPT  },
+        { NULL,          0,                 NULL, 0 },
+    };
 
     weechat_argv0 = (argv[0]) ? strdup (argv[0]) : NULL;
     weechat_upgrading = 0;
     weechat_home = NULL;
+    weechat_home_temp = 0;
+    weechat_home_delete_on_exit = 0;
     weechat_server_cmd_line = 0;
     weechat_force_plugin_autoload = NULL;
     weechat_plugin_no_dlclose = 0;
 
-    for (i = 1; i < argc; i++)
+    optind = 0;
+    opterr = 0;
+
+    while ((opt = getopt_long (argc, argv,
+                               ":acd:thlpP:r:sv",
+                               long_options, NULL)) != -1)
     {
-        if ((strcmp (argv[i], "-c") == 0)
-            || (strcmp (argv[i], "--colors") == 0))
+        switch (opt)
         {
-            gui_color_display_terminal_colors ();
-            weechat_shutdown (EXIT_SUCCESS, 0);
-        }
-        else if ((strcmp (argv[i], "-d") == 0)
-            || (strcmp (argv[i], "--dir") == 0))
-        {
-            if (i + 1 < argc)
-            {
+            case 'a': /* -a / --no-connect */
+                /* option ignored, it will be used by plugins/scripts */
+                break;
+            case 'c': /* -c / --colors */
+                gui_color_display_terminal_colors ();
+                weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case 'd': /* -d / --dir */
+                weechat_home_temp = 0;
                 if (weechat_home)
                     free (weechat_home);
-                weechat_home = strdup (argv[++i]);
-            }
-            else
-            {
-                string_fprintf (stderr,
-                                _("Error: missing argument for \"%s\" option\n"),
-                                argv[i]);
-                weechat_shutdown (EXIT_FAILURE, 0);
-            }
-        }
-        else if ((strcmp (argv[i], "-h") == 0)
-                || (strcmp (argv[i], "--help") == 0))
-        {
-            weechat_display_usage ();
-            weechat_shutdown (EXIT_SUCCESS, 0);
-        }
-        else if ((strcmp (argv[i], "-l") == 0)
-                 || (strcmp (argv[i], "--license") == 0))
-        {
-            weechat_display_copyright ();
-            string_fprintf (stdout, "\n");
-            string_fprintf (stdout, "%s%s", WEECHAT_LICENSE_TEXT);
-            weechat_shutdown (EXIT_SUCCESS, 0);
-        }
-        else if (strcmp (argv[i], "--no-dlclose") == 0)
-        {
-            /*
-             * Valgrind works better when dlclose() is not done after plugins
-             * are unloaded, it can display stack for plugins,* otherwise
-             * you'll see "???" in stack for functions of unloaded plugins.
-             * This option disables the call to dlclose(),
-             * it must NOT be used for other purposes!
-             */
-            weechat_plugin_no_dlclose = 1;
-        }
-        else if (strcmp (argv[i], "--no-gnutls") == 0)
-        {
-            /*
-             * Electric-fence is not working fine when gnutls loads
-             * certificates and Valgrind reports many memory errors with
-             * gnutls.
-             * This option disables the init/deinit of gnutls,
-             * it must NOT be used for other purposes!
-             */
-            weechat_no_gnutls = 1;
-        }
-        else if (strcmp (argv[i], "--no-gcrypt") == 0)
-        {
-            /*
-             * Valgrind reports many memory errors with gcrypt.
-             * This option disables the init/deinit of gcrypt,
-             * it must NOT be used for other purposes!
-             */
-            weechat_no_gcrypt = 1;
-        }
-        else if ((strcmp (argv[i], "-p") == 0)
-                 || (strcmp (argv[i], "--no-plugin") == 0))
-        {
-            if (weechat_force_plugin_autoload)
-                free (weechat_force_plugin_autoload);
-            weechat_force_plugin_autoload = strdup ("!*");
-        }
-        else if ((strcmp (argv[i], "-P") == 0)
-                 || (strcmp (argv[i], "--plugins") == 0))
-        {
-            if (i + 1 < argc)
-            {
+                weechat_home = strdup (optarg);
+                break;
+            case 't': /* -t / --temp-dir */
+                weechat_home_temp = 1;
+                if (weechat_home)
+                {
+                    free (weechat_home);
+                    weechat_home = NULL;
+                }
+                break;
+            case 'h': /* -h / --help */
+                weechat_display_usage ();
+                weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case 'l': /* -l / --license */
+                weechat_display_copyright ();
+                string_fprintf (stdout, "\n");
+                string_fprintf (stdout, "%s%s", WEECHAT_LICENSE_TEXT);
+                weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case 'p': /* -p / --no-plugin */
                 if (weechat_force_plugin_autoload)
                     free (weechat_force_plugin_autoload);
-                weechat_force_plugin_autoload = strdup (argv[++i]);
-            }
-            else
-            {
-                string_fprintf (stderr,
-                                _("Error: missing argument for \"%s\" option\n"),
-                                argv[i]);
-                weechat_shutdown (EXIT_FAILURE, 0);
-            }
-        }
-        else if ((strcmp (argv[i], "-r") == 0)
-                 || (strcmp (argv[i], "--run-command") == 0))
-        {
-            if (i + 1 < argc)
-            {
+                weechat_force_plugin_autoload = strdup ("!*");
+                break;
+            case 'P': /* -P / --plugins */
+                if (weechat_force_plugin_autoload)
+                    free (weechat_force_plugin_autoload);
+                weechat_force_plugin_autoload = strdup (optarg);
+                break;
+            case 'r': /* -r / --run-command */
                 if (!weechat_startup_commands)
                     weechat_startup_commands = weelist_new ();
-                weelist_add (weechat_startup_commands, argv[++i],
+                weelist_add (weechat_startup_commands, optarg,
                              WEECHAT_LIST_POS_END, NULL);
-            }
-            else
-            {
+                break;
+            case 's': /* -s / --no-script */
+                /* option ignored, it will be used by the scripting plugins */
+                break;
+            case 'u': /* --upgrade */
+                weechat_upgrading = 1;
+                break;
+            case 'v': /* -v / --version */
+                string_fprintf (stdout, version_get_version ());
+                fprintf (stdout, "\n");
+                weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case OPTION_NO_DLCLOSE: /* --no-dlclose */
+                /*
+                 * Valgrind works better when dlclose() is not done after
+                 * plugins are unloaded, it can display stack for plugins,*
+                 * otherwise you'll see "???" in stack for functions of
+                 * unloaded plugins.
+                 * This option disables the call to dlclose(),
+                 * it must NOT be used for other purposes!
+                 */
+                weechat_plugin_no_dlclose = 1;
+                break;
+            case OPTION_NO_GNUTLS: /* --no-gnutls */
+                /*
+                 * Electric-fence is not working fine when gnutls loads
+                 * certificates and Valgrind reports many memory errors with
+                 * gnutls.
+                 * This option disables the init/deinit of gnutls,
+                 * it must NOT be used for other purposes!
+                 */
+                weechat_no_gnutls = 1;
+                break;
+            case OPTION_NO_GCRYPT: /* --no-gcrypt */
+                /*
+                 * Valgrind reports many memory errors with gcrypt.
+                 * This option disables the init/deinit of gcrypt,
+                 * it must NOT be used for other purposes!
+                 */
+                weechat_no_gcrypt = 1;
+                break;
+            case ':':
                 string_fprintf (stderr,
                                 _("Error: missing argument for \"%s\" option\n"),
-                                argv[i]);
+                                argv[optind - 1]);
                 weechat_shutdown (EXIT_FAILURE, 0);
-            }
-        }
-        else if (strcmp (argv[i], "--upgrade") == 0)
-        {
-            weechat_upgrading = 1;
-        }
-        else if ((strcmp (argv[i], "-v") == 0)
-                 || (strcmp (argv[i], "--version") == 0))
-        {
-            string_fprintf (stdout, version_get_version ());
-            fprintf (stdout, "\n");
-            weechat_shutdown (EXIT_SUCCESS, 0);
+                break;
+            case '?':
+                /* ignore any unknown option; plugins can use them */
+                break;
+            default:
+                /* ignore any other error */
+                break;
         }
     }
 }
@@ -382,9 +413,60 @@ weechat_set_home_path (char *home_path)
 void
 weechat_create_home_dir ()
 {
-    char *ptr_weechat_home, *config_weechat_home;
+    char *temp_dir, *temp_home_template, *ptr_weechat_home;
+    char *config_weechat_home;
+    int length, add_separator;
     struct stat statinfo;
 
+    /* temporary WeeChat home */
+    if (weechat_home_temp)
+    {
+        temp_dir = util_get_temp_dir ();
+        if (!temp_dir || !temp_dir[0])
+        {
+            string_fprintf (stderr,
+                            _("Error: not enough memory for home "
+                              "directory\n"));
+            weechat_shutdown (EXIT_FAILURE, 0);
+            /* make C static analyzer happy (never executed) */
+            return;
+        }
+        length = strlen (temp_dir) + 32 + 1;
+        temp_home_template = malloc (length);
+        if (!temp_home_template)
+        {
+            free (temp_dir);
+            string_fprintf (stderr,
+                            _("Error: not enough memory for home "
+                              "directory\n"));
+            weechat_shutdown (EXIT_FAILURE, 0);
+            /* make C static analyzer happy (never executed) */
+            return;
+        }
+        add_separator = (temp_dir[strlen (temp_dir) - 1] != DIR_SEPARATOR_CHAR);
+        snprintf (temp_home_template, length,
+                  "%s%sweechat_temp_XXXXXX",
+                  temp_dir,
+                  add_separator ? DIR_SEPARATOR : "");
+        free (temp_dir);
+        ptr_weechat_home = mkdtemp (temp_home_template);
+        if (!ptr_weechat_home)
+        {
+            string_fprintf (stderr,
+                            _("Error: unable to create a temporary "
+                              "home directory (using template: \"%s\")\n"),
+                            temp_home_template);
+            free (temp_home_template);
+            weechat_shutdown (EXIT_FAILURE, 0);
+            /* make C static analyzer happy (never executed) */
+            return;
+        }
+        weechat_home = strdup (ptr_weechat_home);
+        free (temp_home_template);
+        weechat_home_delete_on_exit = 1;
+        return;
+    }
+
     /*
      * weechat_home is not set yet: look for environment variable
      * "WEECHAT_HOME"
@@ -636,6 +718,12 @@ weechat_shutdown (int return_code, int crash)
     network_end ();
     debug_end ();
 
+    if (!crash && weechat_home_delete_on_exit)
+    {
+        /* remove temporary home (only if not crashing) */
+        util_rmtree (weechat_home);
+    }
+
     if (weechat_argv0)
         free (weechat_argv0);
     if (weechat_home)
@@ -702,7 +790,9 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
     completion_init ();                 /* add core completion hooks        */
     gui_key_init ();                    /* init keys                        */
     network_init_gcrypt ();             /* init gcrypt                      */
-    if (!secure_init ())                /* init secured data options (sec.*)*/
+    if (!secure_init ())                /* init secured data                */
+        weechat_shutdown (EXIT_FAILURE, 0);
+    if (!secure_config_init ())         /* init secured data options (sec.*)*/
         weechat_shutdown (EXIT_FAILURE, 0);
     if (!config_weechat_init ())        /* init WeeChat options (weechat.*) */
         weechat_shutdown (EXIT_FAILURE, 0);
@@ -710,7 +800,7 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
     weechat_create_home_dir ();         /* create WeeChat home directory    */
     log_init ();                        /* init log file                    */
     plugin_api_init ();                 /* create some hooks (info,hdata,..)*/
-    secure_read ();                     /* read secured data options        */
+    secure_config_read ();              /* read secured data options        */
     config_weechat_read ();             /* read WeeChat options             */
     network_init_gnutls ();             /* init GnuTLS                      */
 
@@ -719,8 +809,10 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
 
     if (weechat_upgrading)
     {
-        upgrade_weechat_load ();        /* upgrade with session file        */
-        weechat_upgrade_count++;        /* increase /upgrade count          */
+        if (upgrade_weechat_load ())    /* upgrade with session file        */
+            weechat_upgrade_count++;    /* increase /upgrade count          */
+        else
+            weechat_upgrading = 0;
     }
     weechat_startup_message ();         /* display WeeChat startup message  */
     gui_chat_print_lines_waiting_buffer (NULL); /* display lines waiting    */
@@ -747,14 +839,14 @@ weechat_end (void (*gui_end_cb)(int clean_exit))
     plugin_end ();                      /* end plugin interface(s)          */
     if (CONFIG_BOOLEAN(config_look_save_config_on_exit))
         (void) config_weechat_write (); /* save WeeChat config file         */
-    (void) secure_write ();             /* save secured data                */
+    (void) secure_config_write ();      /* save secured data                */
 
     if (gui_end_cb)
         (*gui_end_cb) (1);              /* shut down WeeChat GUI            */
 
     proxy_free_all ();                  /* free all proxies                 */
     config_weechat_free ();             /* free WeeChat options             */
-    secure_free ();                     /* free secured data options        */
+    secure_config_free ();              /* free secured data options        */
     config_file_free_all ();            /* free all configuration files     */
     gui_key_end ();                     /* remove all keys                  */
     unhook_all ();                      /* remove all hooks                 */
