@@ -1,7 +1,7 @@
 /*
  * gui-bar-window.c - bar window functions (used by all GUI)
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -843,6 +843,7 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
                             split_items[i][sub] = string_split (
                                 ptr_content,
                                 "\n",
+                                NULL,
                                 WEECHAT_STRING_SPLIT_STRIP_LEFT
                                 | WEECHAT_STRING_SPLIT_STRIP_RIGHT
                                 | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
@@ -922,6 +923,7 @@ gui_bar_window_content_get_with_filling (struct t_gui_bar_window *bar_window,
                     if (content)
                         free (content);
                     free (split_items);
+                    free (linear_items);
                     return NULL;
                 }
                 content = content2;
