@@ -1,7 +1,7 @@
 /*
  * gui-completion.c - word completion according to context (used by all GUI)
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -562,6 +562,7 @@ gui_completion_get_matching_template (struct t_gui_completion *completion,
     {
         items = string_split (HOOK_COMMAND(hook_command, cplt_templates_static)[i],
                               "|",
+                              NULL,
                               WEECHAT_STRING_SPLIT_STRIP_LEFT
                               | WEECHAT_STRING_SPLIT_STRIP_RIGHT
                               | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
@@ -1319,6 +1320,8 @@ gui_completion_search (struct t_gui_completion *completion, int direction,
     {
         case GUI_COMPLETION_NULL:
             /* should never be executed */
+            if (old_word_found)
+                free (old_word_found);
             return;
         case GUI_COMPLETION_COMMAND:
             gui_completion_command (completion);
