@@ -1,7 +1,7 @@
 /*
  * buflist-bar-item.c - bar item for buflist plugin
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -292,7 +292,6 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
 
     /* make C compiler happy */
     (void) data;
-    (void) window;
     (void) buffer;
     (void) extra_info;
 
@@ -308,6 +307,7 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
     item_index = (int)((unsigned long)pointer);
 
     weechat_hashtable_set (buflist_hashtable_pointers, "bar_item", item);
+    weechat_hashtable_set (buflist_hashtable_pointers, "window", window);
 
     ptr_format = buflist_config_format_buffer_eval;
     ptr_format_current = buflist_config_format_buffer_current_eval;
