@@ -1,7 +1,7 @@
 /*
  * buflist.c - Bar with list of buffers
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -29,6 +29,7 @@
 #include "buflist-bar-item.h"
 #include "buflist-command.h"
 #include "buflist-config.h"
+#include "buflist-info.h"
 #include "buflist-mouse.h"
 
 
@@ -506,6 +507,8 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     weechat_hook_signal ("perl_script_loaded",
                          &buflist_script_loaded_cb, NULL, NULL);
 
+    buflist_info_init ();
+
     return WEECHAT_RC_OK;
 }
 
