@@ -1,7 +1,7 @@
 /*
  * weechat-guile.c - guile (scheme) plugin for WeeChat
  *
- * Copyright (C) 2011-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2011-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -30,6 +30,8 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <libgen.h>
+#include <stdio.h>
+#include <string.h>
 
 #include "../weechat-plugin.h"
 #include "../plugin-script.h"
