@@ -1,7 +1,7 @@
 /*
  * irc-channel.c - channel and private chat management for IRC plugin
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -557,9 +557,13 @@ irc_channel_set_buffer_title (struct t_irc_channel *channel)
             channel->topic,
             (weechat_config_boolean (irc_config_look_topic_strip_colors)) ? 0 : 1);
         weechat_buffer_set (channel->buffer, "title", title_color);
+        if (title_color)
+            free (title_color);
     }
     else
+    {
         weechat_buffer_set (channel->buffer, "title", "");
+    }
 }
 
 /*
@@ -1318,7 +1322,6 @@ irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
     weechat_hashtable_remove (channel->join_smart_filtered, nick);
 }
 
-
 /*
  * Rejoins a channel (for example after kick).
  */
