@@ -1,7 +1,7 @@
 /*
  * irc-color.c - IRC color decoding/encoding in messages
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -95,23 +95,17 @@ regex_t *irc_color_regex_ansi = NULL;
 char *
 irc_color_decode (const char *string, int keep_colors)
 {
-    unsigned char *out, *out2, *ptr_string;
-    int out_length, length, out_pos, length_to_add;
+    char **out, *result;
     char str_fg[3], str_bg[3], str_color[128], str_key[128], str_to_add[128];
     const char *remapped_color;
-    int fg, bg, bold, reverse, italic, underline, rc;
+    unsigned char *ptr_string;
+    int length, fg, bg, bold, reverse, italic, underline, rc;
 
     if (!string)
         return NULL;
 
-    /*
-     * create output string with size of length*2 (with min 128 bytes),
-     * this string will be realloc() later with a larger size if needed
-     */
-    out_length = (strlen (string) * 2) + 1;
-    if (out_length < 128)
-        out_length = 128;
-    out = malloc (out_length);
+    length = strlen (string);
+    out = weechat_string_dyn_alloc (length + (length / 2) + 1);
     if (!out)
         return NULL;
 
@@ -122,8 +116,6 @@ irc_color_decode (const char *string, int keep_colors)
     underline = 0;
 
     ptr_string = (unsigned char *)string;
-    out[0] = '\0';
-    out_pos = 0;
     while (ptr_string && ptr_string[0])
     {
         str_to_add[0] = '\0';
@@ -274,25 +266,13 @@ irc_color_decode (const char *string, int keep_colors)
         }
         /* add "str_to_add" (if not empty) to "out" */
         if (str_to_add[0])
-        {
-            /* if "out" is too small for adding "str_to_add", do a realloc() */
-            length_to_add = strlen (str_to_add);
-            if (out_pos + length_to_add + 1 > out_length)
-            {
-                /* try to double the size of "out" */
-                out_length *= 2;
-                out2 = realloc (out, out_length);
-                if (!out2)
-                    return (char *)out;
-                out = out2;
-            }
-            /* add "str_to_add" to "out" */
-            memcpy (out + out_pos, str_to_add, length_to_add + 1);
-            out_pos += length_to_add;
-        }
+            weechat_string_dyn_concat (out, str_to_add);
     }
 
-    return (char *)out;
+    result = *out;
+    weechat_string_dyn_free (out, 0);
+
+    return result;
 }
 
 /*
@@ -307,58 +287,74 @@ irc_color_decode (const char *string, int keep_colors)
 char *
 irc_color_encode (const char *string, int keep_colors)
 {
-    unsigned char *out, *ptr_string;
-    int out_length, out_pos, length;
+    char **out, *result, str_to_add[2], utf_char[16];
+    unsigned char *ptr_string;
+    int length;
 
     if (!string)
         return NULL;
 
-    out_length = (strlen (string) * 2) + 1;
-    out = malloc (out_length);
+    length = strlen (string);
+    out = weechat_string_dyn_alloc (length + (length / 2) + 1);
     if (!out)
         return NULL;
 
     ptr_string = (unsigned char *)string;
-    out_pos = 0;
-    while (ptr_string && ptr_string[0] && (out_pos < out_length - 1))
+    while (ptr_string && ptr_string[0])
     {
         switch (ptr_string[0])
         {
             case 0x02: /* ^B */
                 if (keep_colors)
-                    out[out_pos++] = IRC_COLOR_BOLD_CHAR;
+                    weechat_string_dyn_concat (out, IRC_COLOR_BOLD_STR);
                 ptr_string++;
                 break;
             case 0x03: /* ^C */
                 if (keep_colors)
-                    out[out_pos++] = IRC_COLOR_COLOR_CHAR;
+                    weechat_string_dyn_concat (out, IRC_COLOR_COLOR_STR);
                 ptr_string++;
                 if (isdigit (ptr_string[0]))
                 {
                     if (keep_colors)
-                        out[out_pos++] = ptr_string[0];
+                    {
+                        str_to_add[0] = ptr_string[0];
+                        str_to_add[1] = '\0';
+                        weechat_string_dyn_concat (out, str_to_add);
+                    }
                     ptr_string++;
                     if (isdigit (ptr_string[0]))
                     {
                         if (keep_colors)
-                            out[out_pos++] = ptr_string[0];
+                        {
+                            str_to_add[0] = ptr_string[0];
+                            str_to_add[1] = '\0';
+                            weechat_string_dyn_concat (out, str_to_add);
+                        }
                         ptr_string++;
                     }
                 }
                 if (ptr_string[0] == ',')
                 {
                     if (keep_colors)
-                        out[out_pos++] = ',';
+                        weechat_string_dyn_concat (out, ",");
                     ptr_string++;
                     if (isdigit (ptr_string[0]))
                     {
                         if (keep_colors)
-                            out[out_pos++] = ptr_string[0];
+                        {
+                            str_to_add[0] = ptr_string[0];
+                            str_to_add[1] = '\0';
+                            weechat_string_dyn_concat (out, str_to_add);
+                        }
                         ptr_string++;
                         if (isdigit (ptr_string[0]))
                         {
                             if (keep_colors)
-                                out[out_pos++] = ptr_string[0];
+                            {
+                                str_to_add[0] = ptr_string[0];
+                                str_to_add[1] = '\0';
+                                weechat_string_dyn_concat (out, str_to_add);
+                            }
                             ptr_string++;
                         }
                     }
@@ -366,37 +362,39 @@ irc_color_encode (const char *string, int keep_colors)
                 break;
             case 0x0F: /* ^O */
                 if (keep_colors)
-                    out[out_pos++] = IRC_COLOR_RESET_CHAR;
+                    weechat_string_dyn_concat (out, IRC_COLOR_RESET_STR);
                 ptr_string++;
                 break;
             case 0x16: /* ^V */
                 if (keep_colors)
-                    out[out_pos++] = IRC_COLOR_REVERSE_CHAR;
+                    weechat_string_dyn_concat (out, IRC_COLOR_REVERSE_STR);
                 ptr_string++;
                 break;
             case 0x1D: /* ^] */
                 if (keep_colors)
-                    out[out_pos++] = IRC_COLOR_ITALIC_CHAR;
+                    weechat_string_dyn_concat (out, IRC_COLOR_ITALIC_STR);
                 ptr_string++;
                 break;
             case 0x1F: /* ^_ */
                 if (keep_colors)
-                    out[out_pos++] = IRC_COLOR_UNDERLINE_CHAR;
+                    weechat_string_dyn_concat (out, IRC_COLOR_UNDERLINE_STR);
                 ptr_string++;
                 break;
             default:
                 length = weechat_utf8_char_size ((char *)ptr_string);
                 if (length == 0)
                     length = 1;
-                memcpy (out + out_pos, ptr_string, length);
-                out_pos += length;
+                memcpy (utf_char, ptr_string, length);
+                utf_char[length] = '\0';
+                weechat_string_dyn_concat (out, utf_char);
                 ptr_string += length;
         }
     }
 
-    out[out_pos] = '\0';
+    result = *out;
+    weechat_string_dyn_free (out, 0);
 
-    return (char *)out;
+    return result;
 }
 
 /*
@@ -485,7 +483,7 @@ irc_color_decode_ansi_cb (void *data, const char *text)
 {
     struct t_irc_color_ansi_state *ansi_state;
     char *text2, **items, *output, str_color[128];
-    int i, length, num_items, value, color;
+    int i, length, num_items, value, value2, color;
 
     ansi_state = (struct t_irc_color_ansi_state *)data;
 
@@ -596,41 +594,43 @@ irc_color_decode_ansi_cb (void *data, const char *text)
             case 38: /* text color */
                 if (i + 1 < num_items)
                 {
-                    switch (atoi (items[i + 1]))
+                    value2 = atoi (items[i + 1]);
+                    if (value2 == 2)
                     {
-                        case 2: /* RGB color */
-                            if (i + 4 < num_items)
+                        /* RGB color */
+                        if (i + 4 < num_items)
+                        {
+                            color = irc_color_convert_rgb2irc (
+                                (atoi (items[i + 2]) << 16) |
+                                (atoi (items[i + 3]) << 8) |
+                                atoi (items[i + 4]));
+                            if (color >= 0)
                             {
-                                color = irc_color_convert_rgb2irc (
-                                    (atoi (items[i + 2]) << 16) |
-                                    (atoi (items[i + 3]) << 8) |
-                                    atoi (items[i + 4]));
-                                if (color >= 0)
-                                {
-                                    snprintf (str_color, sizeof (str_color),
-                                              "%c%02d",
-                                              IRC_COLOR_COLOR_CHAR,
-                                              color);
-                                    strcat (output, str_color);
-                                }
-                                i += 4;
+                                snprintf (str_color, sizeof (str_color),
+                                          "%c%02d",
+                                          IRC_COLOR_COLOR_CHAR,
+                                          color);
+                                strcat (output, str_color);
                             }
-                            break;
-                        case 5: /* terminal color (0-255) */
-                            if (i + 2 < num_items)
+                            i += 4;
+                        }
+                    }
+                    else if (value2 == 5)
+                    {
+                        /* terminal color (0-255) */
+                        if (i + 2 < num_items)
+                        {
+                            color = irc_color_convert_term2irc (atoi (items[i + 2]));
+                            if (color >= 0)
                             {
-                                color = irc_color_convert_term2irc (atoi (items[i + 2]));
-                                if (color >= 0)
-                                {
-                                    snprintf (str_color, sizeof (str_color),
-                                              "%c%02d",
-                                              IRC_COLOR_COLOR_CHAR,
-                                              color);
-                                    strcat (output, str_color);
-                                }
-                                i += 2;
+                                snprintf (str_color, sizeof (str_color),
+                                          "%c%02d",
+                                          IRC_COLOR_COLOR_CHAR,
+                                          color);
+                                strcat (output, str_color);
                             }
-                            break;
+                            i += 2;
+                        }
                     }
                 }
                 break;
@@ -657,41 +657,43 @@ irc_color_decode_ansi_cb (void *data, const char *text)
             case 48: /* background color */
                 if (i + 1 < num_items)
                 {
-                    switch (atoi (items[i + 1]))
+                    value2 = atoi (items[i + 1]);
+                    if (value2 == 2)
                     {
-                        case 2: /* RGB color */
-                            if (i + 4 < num_items)
+                        /* RGB color */
+                        if (i + 4 < num_items)
+                        {
+                            color = irc_color_convert_rgb2irc (
+                                (atoi (items[i + 2]) << 16) |
+                                (atoi (items[i + 3]) << 8) |
+                                atoi (items[i + 4]));
+                            if (color >= 0)
                             {
-                                color = irc_color_convert_rgb2irc (
-                                    (atoi (items[i + 2]) << 16) |
-                                    (atoi (items[i + 3]) << 8) |
-                                    atoi (items[i + 4]));
-                                if (color >= 0)
-                                {
-                                    snprintf (str_color, sizeof (str_color),
-                                              "%c,%02d",
-                                              IRC_COLOR_COLOR_CHAR,
-                                              color);
-                                    strcat (output, str_color);
-                                }
-                                i += 4;
+                                snprintf (str_color, sizeof (str_color),
+                                          "%c,%02d",
+                                          IRC_COLOR_COLOR_CHAR,
+                                          color);
+                                strcat (output, str_color);
                             }
-                            break;
-                        case 5: /* terminal color (0-255) */
-                            if (i + 2 < num_items)
+                            i += 4;
+                        }
+                    }
+                    else if (value2 == 5)
+                    {
+                        /* terminal color (0-255) */
+                        if (i + 2 < num_items)
+                        {
+                            color = irc_color_convert_term2irc (atoi (items[i + 2]));
+                            if (color >= 0)
                             {
-                                color = irc_color_convert_term2irc (atoi (items[i + 2]));
-                                if (color >= 0)
-                                {
-                                    snprintf (str_color, sizeof (str_color),
-                                              "%c,%02d",
-                                              IRC_COLOR_COLOR_CHAR,
-                                              color);
-                                    strcat (output, str_color);
-                                }
-                                i += 2;
+                                snprintf (str_color, sizeof (str_color),
+                                          "%c,%02d",
+                                          IRC_COLOR_COLOR_CHAR,
+                                          color);
+                                strcat (output, str_color);
                             }
-                            break;
+                            i += 2;
+                        }
                     }
                 }
                 break;
