@@ -1,7 +1,7 @@
 /*
  * irc.c - IRC (Internet Relay Chat) plugin for WeeChat
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -165,6 +165,7 @@ int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
     int i, auto_connect;
+    char *info_auto_connect;
 
     weechat_plugin = plugin;
 
@@ -217,16 +218,17 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 
     irc_bar_item_init ();
 
+    /* check if auto-connect is enabled */
+    info_auto_connect = weechat_info_get ("auto_connect", NULL);
+    auto_connect = (info_auto_connect && (strcmp (info_auto_connect, "1") == 0)) ?
+        1 : 0;
+    if (info_auto_connect)
+        free (info_auto_connect);
+
     /* look at arguments */
-    auto_connect = 1;
     for (i = 0; i < argc; i++)
     {
-        if ((weechat_strcasecmp (argv[i], "-a") == 0)
-            || (weechat_strcasecmp (argv[i], "--no-connect") == 0))
-        {
-            auto_connect = 0;
-        }
-        else if ((weechat_strncasecmp (argv[i], IRC_PLUGIN_NAME, 3) == 0))
+        if ((weechat_strncasecmp (argv[i], IRC_PLUGIN_NAME, 3) == 0))
         {
             if (!irc_server_alloc_with_url (argv[i]))
             {
