@@ -2,7 +2,7 @@
  * weechat-php.c - PHP plugin for WeeChat
  *
  * Copyright (C) 2006-2017 Adam Saponara <as@php.net>
- * Copyright (C) 2017-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2017-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -687,11 +687,9 @@ weechat_php_load (const char *filename, const char *code)
     php_registered_script = NULL;
     php_current_script_filename = filename;
 
-    file_handle.filename = filename;
-    file_handle.free_filename = 0;
+    memset (&file_handle, 0, sizeof (file_handle));
     file_handle.type = ZEND_HANDLE_FILENAME;
-    file_handle.opened_path = NULL;
-    file_handle.handle.fp = NULL;
+    file_handle.filename = filename;
 
     zend_try
     {
