@@ -1,7 +1,7 @@
 /*
  * plugin-api-info.c - extra info functions for plugin API
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -27,13 +27,14 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <time.h>
 
 #include "../core/weechat.h"
 #include "../core/wee-config.h"
+#include "../core/wee-crypto.h"
 #include "../core/wee-hook.h"
 #include "../core/wee-infolist.h"
 #include "../core/wee-proxy.h"
-#include "../core/wee-secure.h"
 #include "../core/wee-string.h"
 #include "../core/wee-url.h"
 #include "../core/wee-util.h"
@@ -315,6 +316,48 @@ plugin_api_info_weechat_upgrading_cb (const void *pointer, void *data,
     return strdup (value);
 }
 
+/*
+ * Returns WeeChat info "weechat_headless".
+ */
+
+char *
+plugin_api_info_weechat_headless_cb (const void *pointer, void *data,
+                                     const char *info_name,
+                                     const char *arguments)
+{
+    char value[32];
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) info_name;
+    (void) arguments;
+
+    snprintf (value, sizeof (value), "%d", weechat_headless);
+    return strdup (value);
+}
+
+/*
+ * Returns WeeChat info "auto_connect".
+ */
+
+char *
+plugin_api_info_auto_connect_cb (const void *pointer, void *data,
+                                 const char *info_name,
+                                 const char *arguments)
+{
+    char value[32];
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) info_name;
+    (void) arguments;
+
+    snprintf (value, sizeof (value), "%d", weechat_auto_connect);
+    return strdup (value);
+}
+
 /*
  * Returns WeeChat info "charset_terminal".
  */
@@ -673,7 +716,8 @@ plugin_api_info_uptime_cb (const void *pointer, void *data,
     if (!arguments || !arguments[0])
     {
         /* return uptime with format: "days:hh:mm:ss" */
-        util_get_uptime (NULL, &days, &hours, &minutes, &seconds);
+        util_get_time_diff (weechat_first_start_time, time (NULL),
+                            NULL, &days, &hours, &minutes, &seconds);
         snprintf (value, sizeof (value), "%d:%02d:%02d:%02d",
                   days, hours, minutes, seconds);
         return strdup (value);
@@ -682,7 +726,8 @@ plugin_api_info_uptime_cb (const void *pointer, void *data,
     if (strcmp (arguments, "days") == 0)
     {
         /* return the number of days */
-        util_get_uptime (NULL, &days, NULL, NULL, NULL);
+        util_get_time_diff (weechat_first_start_time, time (NULL),
+                            NULL, &days, NULL, NULL, NULL);
         snprintf (value, sizeof (value), "%d", days);
         return strdup (value);
     }
@@ -690,7 +735,8 @@ plugin_api_info_uptime_cb (const void *pointer, void *data,
     if (strcmp (arguments, "seconds") == 0)
     {
         /* return the number of seconds */
-        util_get_uptime (&total_seconds, NULL, NULL, NULL, NULL);
+        util_get_time_diff (weechat_first_start_time, time (NULL),
+                            &total_seconds, NULL, NULL, NULL, NULL);
         snprintf (value, sizeof (value), "%lld", (long long)total_seconds);
         return strdup (value);
     }
@@ -755,7 +801,7 @@ plugin_api_info_totp_generate_cb (const void *pointer, void *data,
         digits = number;
     }
 
-    totp = secure_totp_generate (ptr_secret, totp_time, digits);
+    totp = weecrypto_totp_generate (ptr_secret, totp_time, digits);
     if (!totp)
         goto error;
 
@@ -828,7 +874,7 @@ plugin_api_info_totp_validate_cb (const void *pointer, void *data,
         window = number;
     }
 
-    rc = secure_totp_validate (ptr_secret, totp_time, window, ptr_otp);
+    rc = weecrypto_totp_validate (ptr_secret, totp_time, window, ptr_otp);
 
     snprintf (value, sizeof (value), "%d", rc);
 
@@ -1736,6 +1782,14 @@ plugin_api_info_init ()
     hook_info (NULL, "weechat_upgrading",
                N_("1 if WeeChat is upgrading (command `/upgrade`)"),
                NULL, &plugin_api_info_weechat_upgrading_cb, NULL, NULL);
+    hook_info (NULL, "weechat_headless",
+               N_("1 if WeeChat is running headless"),
+               NULL, &plugin_api_info_weechat_headless_cb, NULL, NULL);
+    hook_info (NULL, "auto_connect",
+               N_("1 if automatic connection to servers is enabled, "
+                  "0 if it has been disabled by the user (option \"-a\" or "
+                  "\"--no-connect\")"),
+               NULL, &plugin_api_info_auto_connect_cb, NULL, NULL);
     hook_info (NULL, "charset_terminal",
                N_("terminal charset"),
                NULL, &plugin_api_info_charset_terminal_cb, NULL, NULL);
