@@ -1,7 +1,7 @@
 /*
  * plugin-api.c - extra functions for plugin API
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -25,9 +25,11 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <gcrypt.h>
 
 #include "../core/weechat.h"
 #include "../core/wee-config.h"
+#include "../core/wee-crypto.h"
 #include "../core/wee-hashtable.h"
 #include "../core/wee-hook.h"
 #include "../core/wee-infolist.h"
@@ -134,6 +136,71 @@ plugin_api_string_base_decode (int base, const char *from, char *to)
     return -1;
 }
 
+/*
+ * Computes hash of data using the given algorithm.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+plugin_api_crypto_hash (const void *data, int data_size, const char *hash_algo,
+                        void *hash, int *hash_size)
+{
+    int algo;
+
+    if (!hash)
+        return 0;
+
+    if (hash_size)
+        *hash_size = 0;
+
+    if (!data || (data_size < 1) || !hash_algo)
+        return 0;
+
+    algo = weecrypto_get_hash_algo (hash_algo);
+    if (algo == GCRY_MD_NONE)
+        return 0;
+
+    return weecrypto_hash (data, data_size, algo, hash, hash_size);
+}
+
+/*
+ * Computes PKCS#5 Passphrase Based Key Derivation Function number 2 (PBKDF2)
+ * hash of data.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+plugin_api_crypto_hash_pbkdf2 (const void *data, int data_size,
+                               const char *hash_algo,
+                               const void *salt, int salt_size,
+                               int iterations,
+                               void *hash, int *hash_size)
+{
+    int algo;
+
+    if (!hash)
+        return 0;
+
+    if (hash_size)
+        *hash_size = 0;
+
+    if (!data || (data_size < 1) || !hash_algo || !salt || (salt_size < 1))
+        return 0;
+
+    algo = weecrypto_get_hash_algo (hash_algo);
+    if (algo == GCRY_MD_NONE)
+        return 0;
+
+    return weecrypto_hash_pbkdf2 (data, data_size, algo, salt, salt_size,
+                                  iterations, hash, hash_size);
+}
+
 /*
  * Frees an option.
  */
