@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -33,6 +33,16 @@ extern int plugin_api_string_base_encode (int base, const char *from,
 extern int plugin_api_string_base_decode (int base, const char *from,
                                           char *to);
 
+/* crypto */
+extern int plugin_api_crypto_hash (const void *data, int data_size,
+                                   const char *hash_algo,
+                                   void *hash, int *hash_size);
+extern int plugin_api_crypto_hash_pbkdf2 (const void *data, int data_size,
+                                          const char *hash_algo,
+                                          const void *salt, int salt_size,
+                                          int iterations,
+                                          void *hash, int *hash_size);
+
 /* config */
 extern void plugin_api_config_file_option_free (struct t_config_option *option);
 extern struct t_config_option *plugin_api_config_get (const char *option_name);
