@@ -1,7 +1,7 @@
 /*
  * relay-raw.c - functions for Relay raw data messages
  *
- * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -285,7 +285,8 @@ relay_raw_message_add (struct t_relay_client *client,
 
     if (!(flags & RELAY_RAW_FLAG_BINARY)
         || (msg_type == RELAY_CLIENT_MSG_PING)
-        || (msg_type == RELAY_CLIENT_MSG_PONG))
+        || (msg_type == RELAY_CLIENT_MSG_PONG)
+        || (msg_type == RELAY_CLIENT_MSG_CLOSE))
     {
         /* build prefix with arrow */
         prefix_arrow[0] = '\0';
