@@ -1,7 +1,7 @@
 /*
  * relay-websocket.c - websocket server functions for relay plugin (RFC 6455)
  *
- * Copyright (C) 2013-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2013-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -23,7 +23,6 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
-#include <gcrypt.h>
 
 #include "../weechat-plugin.h"
 #include "relay.h"
@@ -188,10 +187,8 @@ char *
 relay_websocket_build_handshake (struct t_relay_client *client)
 {
     const char *sec_websocket_key;
-    char *key, sec_websocket_accept[128], handshake[1024];
-    unsigned char *result;
-    gcry_md_hd_t hd;
-    int length;
+    char *key, sec_websocket_accept[128], handshake[1024], hash[160 / 8];
+    int length, hash_size;
 
     sec_websocket_key = weechat_hashtable_get (client->http_headers,
                                                "sec-websocket-key");
@@ -210,16 +207,16 @@ relay_websocket_build_handshake (struct t_relay_client *client)
     snprintf (key, length, "%s%s", sec_websocket_key, WEBSOCKET_GUID);
 
     /* compute 160-bit SHA1 on the key and encode it with base64 */
-    gcry_md_open (&hd, GCRY_MD_SHA1, 0);
-    length = gcry_md_get_algo_dlen (GCRY_MD_SHA1);
-    gcry_md_write (hd, key, strlen (key));
-    result = gcry_md_read (hd, GCRY_MD_SHA1);
-    if (weechat_string_base_encode (64, (char *)result, length,
+    if (!weechat_crypto_hash (key, strlen (key), "sha1", hash, &hash_size))
+    {
+        free (key);
+        return NULL;
+    }
+    if (weechat_string_base_encode (64, hash, hash_size,
                                     sec_websocket_accept) < 0)
     {
         sec_websocket_accept[0] = '\0';
     }
-    gcry_md_close (hd);
 
     free (key);
 
