@@ -1,7 +1,7 @@
 /*
  * weechat-ruby.c - ruby plugin for WeeChat
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005-2007 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -178,7 +178,7 @@ weechat_ruby_hashtable_to_hash (struct t_hashtable *hashtable)
  */
 
 int
-weechat_ruby_hash_foreach_cb (VALUE key, VALUE value, void *arg)
+weechat_ruby_hash_foreach_cb (VALUE key, VALUE value, VALUE arg)
 {
     struct t_hashtable *hashtable;
     const char *type_values;
