@@ -1,7 +1,7 @@
 /*
  * script-action.c - actions on scripts
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -111,8 +111,6 @@ script_action_list_input (int send_to_buffer, int translated)
     if (!buf)
         return;
 
-    length = 0;
-
     for (i = 0; i < SCRIPT_NUM_LANGUAGES; i++)
     {
         snprintf (hdata_name, sizeof (hdata_name),
@@ -1267,7 +1265,7 @@ script_action_run ()
                     {
                         if (weechat_strcasecmp (argv[1], "-i") == 0)
                             script_action_list_input (0, 0);
-                        if (weechat_strcasecmp (argv[1], "-il") == 0)
+                        else if (weechat_strcasecmp (argv[1], "-il") == 0)
                             script_action_list_input (0, 1);
                         else if (weechat_strcasecmp (argv[1], "-o") == 0)
                             script_action_list_input (1, 0);
