@@ -1,7 +1,7 @@
 /*
  * script-repo.c - download and read repository file (plugins.xml.gz)
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -36,7 +36,6 @@
 #include <stdio.h>
 #include <time.h>
 #include <zlib.h>
-#include <gcrypt.h>
 
 #include "../weechat-plugin.h"
 #include "script.h"
@@ -756,13 +755,8 @@ script_repo_sha512sum_file (const char *filename)
 {
     struct stat st;
     FILE *file;
-    char sha512sum[512];
-    const char *hexa = "0123456789abcdef";
-    unsigned char *data, *result;
-    gcry_md_hd_t hd;
-    int mdlen, i;
-
-    sha512sum[0] = '\0';
+    char *data, hash[512 / 8], hash_hexa[((512 / 8) * 2) + 1];
+    int hash_size;
 
     if (stat (filename, &st) == -1)
         return NULL;
@@ -780,21 +774,17 @@ script_repo_sha512sum_file (const char *filename)
     }
     fclose (file);
 
-    gcry_md_open (&hd, GCRY_MD_SHA512, 0);
-    mdlen = gcry_md_get_algo_dlen (GCRY_MD_SHA512);
-    gcry_md_write (hd, data, st.st_size);
-    result = gcry_md_read (hd, GCRY_MD_SHA512);
-    for (i = 0; i < mdlen; i++)
+    if (!weechat_crypto_hash (data, st.st_size, "sha512", hash, &hash_size))
     {
-        sha512sum[i * 2] = hexa[(result[i] & 0xFF) / 16];
-        sha512sum[(i * 2) + 1] = hexa[(result[i] & 0xFF) % 16];
+        free (data);
+        return NULL;
     }
-    sha512sum[((mdlen - 1) * 2) + 2] = '\0';
-    gcry_md_close (hd);
+    weechat_string_base_encode (16, hash, hash_size, hash_hexa);
+    weechat_string_tolower (hash_hexa);
 
     free (data);
 
-    return strdup (sha512sum);
+    return strdup (hash_hexa);
 }
 
 /*
@@ -1241,6 +1231,8 @@ script_repo_file_read (int quiet)
         {
             if (strstr (ptr_line, "<plugin id="))
             {
+                if (script)
+                    script_repo_free (script);
                 script = script_repo_alloc ();
                 weechat_hashtable_remove_all (descriptions);
             }
