@@ -1,7 +1,7 @@
 /*
  * test-core-eval.cpp - test evaluation functions
  *
- * Copyright (C) 2014-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -30,8 +30,11 @@ extern "C"
 #include "src/core/wee-config.h"
 #include "src/core/wee-config-file.h"
 #include "src/core/wee-hashtable.h"
+#include "src/core/wee-secure.h"
 #include "src/core/wee-string.h"
 #include "src/core/wee-version.h"
+#include "src/gui/gui-buffer.h"
+#include "src/gui/gui-line.h"
 #include "src/gui/gui-color.h"
 #include "src/plugins/plugin.h"
 }
@@ -72,6 +75,7 @@ TEST(CoreEval, Boolean)
 TEST(CoreEval, EvalCondition)
 {
     struct t_hashtable *pointers, *extra_vars, *options;
+    const char *ptr_debug_output;
     char *value;
 
     pointers = NULL;
@@ -96,6 +100,7 @@ TEST(CoreEval, EvalCondition)
     WEE_CHECK_EVAL("0", "");
     WEE_CHECK_EVAL("0", "0");
     WEE_CHECK_EVAL("0", "1 == 2");
+    WEE_CHECK_EVAL("0", "  1 == 2  ");
     WEE_CHECK_EVAL("0", "==1");
     WEE_CHECK_EVAL("0", "1 >= 2");
     WEE_CHECK_EVAL("0", "2 <= 1");
@@ -133,10 +138,14 @@ TEST(CoreEval, EvalCondition)
     WEE_CHECK_EVAL("0", "1 && (0 || 0)");
     WEE_CHECK_EVAL("0", "(0 || 1) && 0");
     WEE_CHECK_EVAL("0", "((0 || 1) && 1) && 0");
+    WEE_CHECK_EVAL("0", "(0) ");
     WEE_CHECK_EVAL("0", "abcd =~ (?-i)^ABC");
     WEE_CHECK_EVAL("0", "abcd =~ \\(abcd\\)");
     WEE_CHECK_EVAL("0", "=~abcd");
     WEE_CHECK_EVAL("0", "(abcd) =~ \\(\\(abcd\\)\\)");
+    WEE_CHECK_EVAL("0", "abcd !~ ^ABC");
+    WEE_CHECK_EVAL("0", "abcd !~ (?-i)^abc");
+    WEE_CHECK_EVAL("0", "abcd!~abc");
     WEE_CHECK_EVAL("0", "abcd =* abce");
     WEE_CHECK_EVAL("0", "abcd =* a*e");
     WEE_CHECK_EVAL("0", "abcd !* *bc*");
@@ -147,12 +156,17 @@ TEST(CoreEval, EvalCondition)
     WEE_CHECK_EVAL("0", "${window.buffer.number} == 2");
     WEE_CHECK_EVAL("0", "${calc:2+3} < 5");
     WEE_CHECK_EVAL("0", "${calc:1.5*3} < 4.5");
+    WEE_CHECK_EVAL("0", "${if:${buffer.number}==2?yes:}");
+    WEE_CHECK_EVAL("0", "${if:${buffer.number}==2?yes:no} == yes");
+    WEE_CHECK_EVAL("0", "yes == ${if:${buffer.number}==2?yes:no}");
+    WEE_CHECK_EVAL("0", "${if:\\$==A?yes:}");
 
     /* conditions evaluated as true */
     WEE_CHECK_EVAL("1", "1");
     WEE_CHECK_EVAL("1", "123");
     WEE_CHECK_EVAL("1", "abc");
     WEE_CHECK_EVAL("1", "2 == 2");
+    WEE_CHECK_EVAL("1", "  2 == 2  ");
     WEE_CHECK_EVAL("1", "==0");
     WEE_CHECK_EVAL("1", "2 >= 1");
     WEE_CHECK_EVAL("1", "1 <= 2");
@@ -189,6 +203,7 @@ TEST(CoreEval, EvalCondition)
     WEE_CHECK_EVAL("1", "1 && (0 || 1)");
     WEE_CHECK_EVAL("1", "(0 || 1) && 1");
     WEE_CHECK_EVAL("1", "((0 || 1) && 1) && 1");
+    WEE_CHECK_EVAL("1", "(1)1");
     WEE_CHECK_EVAL("1", "abcd =~ ^ABC");
     WEE_CHECK_EVAL("1", "abcd =~ (?-i)^abc");
     WEE_CHECK_EVAL("1", "abcd=~abc");
@@ -197,6 +212,9 @@ TEST(CoreEval, EvalCondition)
     WEE_CHECK_EVAL("1", "(abcd) =~ (abcd)");
     WEE_CHECK_EVAL("1", "(abcd) =~ \\(abcd\\)");
     WEE_CHECK_EVAL("1", "((abcd)) =~ \\(\\(abcd\\)\\)");
+    WEE_CHECK_EVAL("1", "abcd !~ (?-i)^ABC");
+    WEE_CHECK_EVAL("1", "abcd !~ \\(abcd\\)");
+    WEE_CHECK_EVAL("1", "!~abcd");
     WEE_CHECK_EVAL("1", "abcd !* abce");
     WEE_CHECK_EVAL("1", "abcd !* a*e");
     WEE_CHECK_EVAL("1", "abcd =* *bc*");
@@ -207,6 +225,10 @@ TEST(CoreEval, EvalCondition)
     WEE_CHECK_EVAL("1", "${window.buffer.number} == 1");
     WEE_CHECK_EVAL("1", "${calc:2+3} >= 5");
     WEE_CHECK_EVAL("1", "${calc:1.5*3} >= 4.5");
+    WEE_CHECK_EVAL("1", "${if:${buffer.number}==1?yes:}");
+    WEE_CHECK_EVAL("1", "${if:${buffer.number}==1?yes:no} == yes");
+    WEE_CHECK_EVAL("1", "yes == ${if:${buffer.number}==1?yes:no}");
+    WEE_CHECK_EVAL("1", "${if:\\$==\\$?yes:}");
 
     /* evaluation of extra_vars */
     hashtable_set (options, "extra", "eval");
@@ -220,6 +242,57 @@ TEST(CoreEval, EvalCondition)
     hashtable_set (options, "suffix", ")%");
     WEE_CHECK_EVAL("0", "${buffer.number} == 1");
     WEE_CHECK_EVAL("1", "%(buffer.number)% == 1");
+    hashtable_remove (options, "prefix");
+    hashtable_remove (options, "suffix");
+
+    /* test with debug */
+    hashtable_set (options, "debug", "1");
+    WEE_CHECK_EVAL("1", "abc < def");
+    ptr_debug_output = (const char *)hashtable_get (options, "debug_output");
+    STRCMP_EQUAL("eval_expression(\"abc < def\")\n"
+                 "eval_expression_condition(\"abc < def\")\n"
+                 "eval_strstr_level(\"abc < def\", \"||\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc < def\", \"&&\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc < def\", \"=~\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc < def\", \"!~\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc < def\", \"=*\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc < def\", \"!*\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc < def\", \"==\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc < def\", \"!=\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc < def\", \"<=\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc < def\", \"<\", \"(\", \")\", 0)\n"
+                 "eval_expression_condition(\"abc\")\n"
+                 "eval_strstr_level(\"abc\", \"||\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \"&&\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \"=~\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \"!~\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \"=*\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \"!*\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \"==\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \"!=\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \"<=\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \"<\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \">=\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"abc\", \">\", \"(\", \")\", 0)\n"
+                 "eval_replace_vars(\"abc\")\n"
+                 "eval_expression_condition(\"def\")\n"
+                 "eval_strstr_level(\"def\", \"||\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \"&&\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \"=~\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \"!~\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \"=*\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \"!*\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \"==\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \"!=\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \"<=\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \"<\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \">=\", \"(\", \")\", 0)\n"
+                 "eval_strstr_level(\"def\", \">\", \"(\", \")\", 0)\n"
+                 "eval_replace_vars(\"def\")\n"
+                 "eval_compare(\"abc\", \"<\", \"def\")",
+                 ptr_debug_output);
+    hashtable_remove (options, "debug");
+    hashtable_remove (options, "debug_output");
 
     hashtable_free (extra_vars);
     hashtable_free (options);
@@ -235,6 +308,7 @@ TEST(CoreEval, EvalExpression)
     struct t_hashtable *pointers, *extra_vars, *options;
     struct t_config_option *ptr_option;
     char *value, str_value[256];
+    const char *ptr_debug_output;
 
     pointers = NULL;
 
@@ -269,9 +343,18 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("\t", "${esc:\t}");
 
     /* test hidden chars */
+    WEE_CHECK_EVAL("", "${hide:invalid}");
     WEE_CHECK_EVAL("********", "${hide:*,password}");
     WEE_CHECK_EVAL("\u2603\u2603\u2603", "${hide:${esc:\u2603},abc}");
 
+    /* test cut of chars (invalid values) */
+    WEE_CHECK_EVAL("", "${cut:}");
+    WEE_CHECK_EVAL("", "${cut:0,}");
+    WEE_CHECK_EVAL("", "${cut:a,,}");
+    WEE_CHECK_EVAL("", "${cutscr:}");
+    WEE_CHECK_EVAL("", "${cutscr:0,}");
+    WEE_CHECK_EVAL("", "${cutscr:a,,}");
+
     /* test cut of chars */
     WEE_CHECK_EVAL("", "${cut:0,,}");
     WEE_CHECK_EVAL("", "${cutscr:0,,}");
@@ -367,6 +450,11 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("Hello, \x19" "F03world!",
                    "${revscr:${revscr:Hello, ${color:red}world!}}");
 
+    /* test repeat of string (invalid values) */
+    WEE_CHECK_EVAL("", "${repeat:}");
+    WEE_CHECK_EVAL("", "${repeat:0}");
+    WEE_CHECK_EVAL("", "${repeat:a,x}");
+
     /* test repeat of string */
     WEE_CHECK_EVAL("", "${repeat:-1,x}");
     WEE_CHECK_EVAL("", "${repeat:0,x}");
@@ -404,6 +492,11 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL(str_value, "${color:irc.color.message_join}-test-");
     WEE_CHECK_EVAL("test", "${option.not.found}test");
 
+    /* test modifier (invalid values) */
+    WEE_CHECK_EVAL("test_", "test_${modifier:}");
+    WEE_CHECK_EVAL("test_", "test_${modifier:xxx}");
+    WEE_CHECK_EVAL("test_", "test_${modifier:xxx,data}");
+
     /* test modifier */
     WEE_CHECK_EVAL("test_string", "test_${modifier:xxx,data,string}");
     WEE_CHECK_EVAL("test_no_color",
@@ -422,6 +515,7 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL(version_get_version (), "${info:version}");
 
     /* test date */
+    WEE_CHECK_EVAL("", "${date:}");
     value = eval_expression ("${date}", pointers, extra_vars, options);
     LONGS_EQUAL(19, strlen (value));
     free (value);
@@ -457,10 +551,21 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("123129", "${calc:${repeat:2,123}+2*3}");
 
     /* test option */
+    hashtable_set (secure_hashtable_data, "sec_option", "sec_value");
+    WEE_CHECK_EVAL("sec_value", "${sec.data.sec_option}");
+    hashtable_remove (secure_hashtable_data, "sec_option");
     snprintf (str_value, sizeof (str_value),
               "%d", CONFIG_INTEGER(config_look_scroll_amount));
     WEE_CHECK_EVAL(str_value, "${weechat.look.scroll_amount}");
     WEE_CHECK_EVAL(str_value, "${${window.buffer.name}.look.scroll_amount}");
+    WEE_CHECK_EVAL("right", "${weechat.look.prefix_align}");
+    WEE_CHECK_EVAL("1", "${weechat.startup.display_logo}");
+    WEE_CHECK_EVAL("=!=", "${weechat.look.prefix_error}");
+    WEE_CHECK_EVAL("lightcyan", "${weechat.color.chat_nick}");
+
+    /* test buffer local variable */
+    WEE_CHECK_EVAL("core", "${plugin}");
+    WEE_CHECK_EVAL("weechat", "${name}");
 
     /* test hdata */
     WEE_CHECK_EVAL("x", "x${buffer.number");
@@ -470,6 +575,25 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("1", "${window.buffer.number}");
     WEE_CHECK_EVAL("core.weechat", "${buffer.full_name}");
     WEE_CHECK_EVAL("core.weechat", "${window.buffer.full_name}");
+    WEE_CHECK_EVAL("", "${buffer[0x0].full_name}");
+    WEE_CHECK_EVAL("core.weechat", "${buffer[gui_buffers].full_name}");
+    snprintf (str_value, sizeof (str_value),
+              "${buffer[0x%lx].full_name}", (long unsigned int)gui_buffers);
+    WEE_CHECK_EVAL("core.weechat", str_value);
+    snprintf (str_value, sizeof (str_value), "%c", 1);
+    WEE_CHECK_EVAL(str_value,
+                   "${window.buffer.own_lines.first_line.data.displayed}");
+    WEE_CHECK_EVAL("1", "${window.buffer.num_displayed}");
+    snprintf (str_value, sizeof (str_value),
+              "%lld",
+              (long long)(gui_buffers->own_lines->first_line->data->date));
+    WEE_CHECK_EVAL(str_value,
+                   "${window.buffer.own_lines.first_line.data.date}");
+    snprintf (str_value, sizeof (str_value),
+              "0x%lx", (long unsigned int)(gui_buffers->local_variables));
+    WEE_CHECK_EVAL(str_value, "${window.buffer.local_variables}");
+    WEE_CHECK_EVAL("core", "${window.buffer.local_variables.plugin}");
+    WEE_CHECK_EVAL("weechat", "${window.buffer.local_variables.name}");
 
     /* test with another prefix/suffix */
     options = hashtable_new (32,
@@ -489,9 +613,24 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("<<info:version>>", "<<info:version>>");
     WEE_CHECK_EVAL(version_get_version (), "<<<info:version>>>");
     WEE_CHECK_EVAL("1", "<<<buffer.number>>>");
+    hashtable_free (options);
+
+    /* test with debug */
+    options = hashtable_new (32,
+                             WEECHAT_HASHTABLE_STRING,
+                             WEECHAT_HASHTABLE_STRING,
+                             NULL, NULL);
+    CHECK(options);
+    hashtable_set (options, "debug", "1");
+    WEE_CHECK_EVAL("fedcba", "${rev:abcdef}");
+    ptr_debug_output = (const char *)hashtable_get (options, "debug_output");
+    STRCMP_EQUAL("eval_expression(\"${rev:abcdef}\")\n"
+                 "eval_replace_vars(\"${rev:abcdef}\")\n"
+                 "eval_replace_vars_cb(\"rev:abcdef\")",
+                 ptr_debug_output);
+    hashtable_free (options);
 
     hashtable_free (extra_vars);
-    hashtable_free (options);
 }
 
 /*
@@ -595,6 +734,12 @@ TEST(CoreEval, EvalReplaceRegex)
                    "${re:0} -- ${re:1} ${re:+} (${re:#})");
     WEE_CHECK_EVAL("abc def ghi jkl -- abc jkl (4)", "abc def ghi jkl");
 
+    /* invalid regex group */
+    hashtable_remove (pointers, "regex");
+    hashtable_set (options, "regex", "abc");
+    hashtable_set (options, "regex_replace", "${re:z}");
+    WEE_CHECK_EVAL("", "abc");
+
     hashtable_free (pointers);
     hashtable_free (extra_vars);
     hashtable_free (options);
