@@ -1,7 +1,7 @@
 /*
  * test-core-util.cpp - test util functions
  *
- * Copyright (C) 2014-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2020 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -28,6 +28,7 @@ extern "C"
 #include <string.h>
 #include <signal.h>
 #include <sys/time.h>
+#include "src/core/wee-string.h"
 #include "src/core/wee-util.h"
 }
 
@@ -101,6 +102,75 @@ TEST(CoreUtil, GetTimeString)
     STRCMP_EQUAL("Sat, 01 Jan 2000 00:00:00", str_date);
 }
 
+/*
+ * Tests functions:
+ *   util_get_time_diff
+ */
+
+TEST(CoreUtil, GetTimeDiff)
+{
+    time_t date1, date2, total_seconds;
+    int days, hours, minutes, seconds;
+
+    util_get_time_diff (0, 0, NULL, NULL, NULL, NULL, NULL);
+
+    date1 = 946684800;  /* 2000-01-01 00:00:00 */
+
+    date2 = 946684800;  /* 2000-01-01 00:00:00 */
+    util_get_time_diff (date1, date2,
+                        &total_seconds, &days, &hours, &minutes, &seconds);
+    LONGS_EQUAL(0, total_seconds);
+    LONGS_EQUAL(0, days);
+    LONGS_EQUAL(0, hours);
+    LONGS_EQUAL(0, minutes);
+    LONGS_EQUAL(0, seconds);
+
+    date2 = 946684801;  /* 2000-01-01 00:00:01 */
+    util_get_time_diff (date1, date2,
+                        &total_seconds, &days, &hours, &minutes, &seconds);
+    LONGS_EQUAL(1, total_seconds);
+    LONGS_EQUAL(0, days);
+    LONGS_EQUAL(0, hours);
+    LONGS_EQUAL(0, minutes);
+    LONGS_EQUAL(1, seconds);
+
+    date2 = 946684880;  /* 2000-01-01 00:01:20 */
+    util_get_time_diff (date1, date2,
+                        &total_seconds, &days, &hours, &minutes, &seconds);
+    LONGS_EQUAL(80, total_seconds);
+    LONGS_EQUAL(0, days);
+    LONGS_EQUAL(0, hours);
+    LONGS_EQUAL(1, minutes);
+    LONGS_EQUAL(20, seconds);
+
+    date2 = 946695680;  /* 2000-01-01 03:01:20 */
+    util_get_time_diff (date1, date2,
+                        &total_seconds, &days, &hours, &minutes, &seconds);
+    LONGS_EQUAL(10880, total_seconds);
+    LONGS_EQUAL(0, days);
+    LONGS_EQUAL(3, hours);
+    LONGS_EQUAL(1, minutes);
+    LONGS_EQUAL(20, seconds);
+
+    date2 = 947127680;  /* 2000-01-06 03:01:20 */
+    util_get_time_diff (date1, date2,
+                        &total_seconds, &days, &hours, &minutes, &seconds);
+    LONGS_EQUAL(442880, total_seconds);
+    LONGS_EQUAL(5, days);
+    LONGS_EQUAL(3, hours);
+    LONGS_EQUAL(1, minutes);
+    LONGS_EQUAL(20, seconds);
+
+    date2 = 979527680;  /* 2001-01-15 03:01:20 */
+    util_get_time_diff (date1, date2,
+                        &total_seconds, &days, &hours, &minutes, &seconds);
+    LONGS_EQUAL(32842880, total_seconds);
+    LONGS_EQUAL(380, days);
+    LONGS_EQUAL(3, hours);
+    LONGS_EQUAL(1, minutes);
+    LONGS_EQUAL(20, seconds);
+}
+
 /*
  * Tests functions:
  *   util_parse_delay
@@ -238,7 +308,48 @@ TEST(CoreUtil, LibName)
 
 TEST(CoreUtil, FileGetContent)
 {
-    /* TODO: write tests */
+    char *path, *content, *content_read;
+    const char *content_small = "line 1\nline 2\nend";
+    int length, i;
+    FILE *f;
+
+    /* file not found */
+    POINTERS_EQUAL(NULL, util_file_get_content (NULL));
+    POINTERS_EQUAL(NULL, util_file_get_content (""));
+    POINTERS_EQUAL(NULL, util_file_get_content ("/tmp/does/not/exist.xyz"));
+
+    path = string_eval_path_home ("%h/test_file.txt", NULL, NULL, NULL);
+
+    /* small file */
+    length = strlen (content_small);
+    f = fopen (path, "wb");
+    CHECK(f);
+    LONGS_EQUAL(length, fwrite (content_small, 1, length, f));
+    fclose (f);
+    content_read = util_file_get_content (path);
+    STRCMP_EQUAL(content_small, content_read);
+    free (content_read);
+    unlink (path);
+
+    /* bigger file: 26 lines of 5000 bytes */
+    length = 26 * 5001;
+    content = (char *)malloc (length + 1);
+    CHECK(content);
+    for (i = 0; i < 26; i++)
+    {
+        memset (content + (i * 5001), 'a' + i, 5000);
+        content[(i * 5001) + 5000] = '\n';
+    }
+    content[26 * 5001] = '\0';
+    f = fopen (path, "wb");
+    CHECK(f);
+    LONGS_EQUAL(length, fwrite (content, 1, length, f));
+    fclose (f);
+    content_read = util_file_get_content (path);
+    STRCMP_EQUAL(content, content_read);
+    free (content_read);
+    unlink (path);
+    free (content);
 }
 
 /*
