@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2015-2019 Sébastien Helleu <flashcode@flashtux.org>
+# Copyright (C) 2015-2020 Sébastien Helleu <flashcode@flashtux.org>
 #
 # This file is part of WeeChat, the extensible chat client.
 #
@@ -32,9 +32,9 @@
 #     devel-patch  the patch version of devel (e.g. 2 for version 1.4.2)
 #
 
-WEECHAT_STABLE=2.7.1
-WEECHAT_DEVEL=2.7.1
-WEECHAT_DEVEL_FULL=2.7.1
+WEECHAT_STABLE=2.8
+WEECHAT_DEVEL=2.8
+WEECHAT_DEVEL_FULL=2.8
 
 if [ $# -lt 1 ]; then
     echo >&2 "Syntax: $0 stable|devel|devel-full|devel-major|devel-minor|devel-patch"
